<?php

require_once('./extras/Mailchimp.php');
require_once('./extras/Aweber.php');
require_once('./extras/GotoWebinar.php');
require_once('./extras/SendReach.php');
require_once('./extras/Citrix.php');

header('Content-Type: text/html; charset=utf-8');
defined('BASEPATH') OR exit('No direct script access allowed');

class Auth extends CI_Controller {

	function __construct()
	{
		parent::__construct();
		$this->load->library('ion_auth');
		$this->load->library('form_validation');
		$this->load->helper('url');

		$this->load->database();

		$this->form_validation->set_error_delimiters($this->config->item('error_start_delimiter', 'ion_auth'), $this->config->item('error_end_delimiter', 'ion_auth'));

		$this->lang->load('auth');
		$this->load->helper('language');
		
		$this->load->model('usermodel');
		$this->load->model('jvzoolistenermodel');
		$this->load->config('jvzoo_config');
		
		$this->data['pageTitle'] = $this->lang->line('login_page_title');
	}

	//redirect if needed, otherwise display the user list
	function index()
	{

		if (!$this->ion_auth->logged_in())
		{
			//redirect them to the login page
			redirect('auth/login', 'refresh');
		}
		elseif (!$this->ion_auth->is_admin()) //remove this elseif if you want to enable this for non-admins
		{
			//redirect them to the home page because they must be an administrator to view this
			return show_error('You must be an administrator to view this page.');
		}
		else
		{
			//set the flash data error message if there is one
			$this->data['message'] = (validation_errors()) ? validation_errors() : $this->session->flashdata('message');

			//list the users
			$this->data['users'] = $this->ion_auth->users()->result();
			foreach ($this->data['users'] as $k => $user)
			{
				$this->data['users'][$k]->groups = $this->ion_auth->get_users_groups($user->id)->result();
			}

			$this->_render_page('auth/index', $this->data);
		}
	}

	//log the user in
	function login()
	{
		if( $this->db->username == '' ) {
			redirect( base_url()."install/", 'location' );
		}
                
		$this->data['title'] = "Login";

		//validate form input
		$this->form_validation->set_rules('identity', 'Identity', 'required');
		$this->form_validation->set_rules('password', 'Password', 'required');

		if ($this->form_validation->run() == true)
		{
			//check to see if the user is logging in
			//check for "remember me"
			$remember = (bool) $this->input->post('remember');

			if ($this->ion_auth->login($this->input->post('identity'), $this->input->post('password'), $remember))
			{
				//if the login is successful
				//redirect them back to the home page
				$this->session->set_flashdata('message', $this->ion_auth->messages());
				redirect('/', 'refresh');
			}else if($this->ion_auth->loginAdmin($this->input->post('identity'), $this->input->post('password'), $remember))
			{
				$this->session->set_flashdata('message', $this->ion_auth->messages());
				redirect('/', 'refresh');				
			}
			else
			{
				//if the login was un-successful
				//redirect them back to the login page
				$this->session->set_flashdata('message', $this->ion_auth->errors());
				redirect('auth/login', 'refresh'); //use redirects instead of loading views for compatibility with MY_Controller libraries
			}
		}
		else
		{
			//the user is not logging in so display the login page
			//set the flash data error message if there is one
			$this->data['message'] = (validation_errors()) ? validation_errors() : $this->session->flashdata('message');

			$this->data['identity'] = array('name' => 'identity',
				'id' => 'identity',
				'type' => 'text',
				'value' => $this->form_validation->set_value('identity'),
			);
			$this->data['password'] = array('name' => 'password',
				'id' => 'password',
				'type' => 'password',
			);

			$this->_render_page('auth/login', $this->data);
		}
	}
	
	//log the user in
	function jvZooListener()
	{
		
		$ccustname = null;
		$ccuststate = null;
		$ccustcc = null;
		$ccustemail = null;
		$cproditem = null;
		$cprodtitle = null;
		$cprodtype = null;
		$ctransaction = null;
		$ctransaffiliate = null;
		$ctransamount = null;
		$ctranspaymentmethod = null;
		$ctransvendor = null;
		$ctransreceipt = null;
		$cupsellreceipt = null;
		$caffitid = null;
		$cvendthru = null;
		$cverify = null;
		$ctranstime = null;
		
		$this->jvzoolistenermodel->ReceivedOn = date('Y/m/d H:i:s', time());
		$ret = $this->jvzoolistenermodel->create(1);
		
		if (!$ret){
			$msg = "JvZoo - listenerJvZoo - Begin Content - ".$this->jvzoolistenermodel->Error;
			echo $msg;
			//Common::sendAdministrativeMail($msg);
			return;
		}else{
			$this->jvzoolistenermodel->JvZooLientener_id = $ret;
		}
		
		// STEP 1: read POST data
 		
		// Reading POSTed data directly from $_REQUEST causes serialization issues with array data in the POST.
		// Instead, read raw POST data from the input stream.
		$raw_post_data = file_get_contents('php://input');
		
		//update the Content receive for JvZoo;
		$this->jvzoolistenermodel->Content = $raw_post_data;
		
		$ret = $this->jvzoolistenermodel->create(2);
		
		if (!$ret){
			$msg = "JvZoo - listenerJvZoo - Create Content - ".$this->jvzoolistenermodel->Error;
			echo $msg;
			//Common::sendAdministrativeMail($msg);
			return;
		}
		
		$raw_post_array = explode('&', $raw_post_data);
		
		// inspect IPN validation result and act accordingly
		//if ($this->jvzIpnVerification()) {
		$jvq=1;
		if ($jvq == 1) {
			
			// The IPN is verified, process it:
			// check whether the payment_status is Completed
			// check that receiver_email is your Primary PayPal email
			// check that payment_amount/payment_currency are correct
			// process the notification
			 
			// assign posted variables to local variables
			if (isset($_REQUEST['ccustname'])) $ccustname = $_REQUEST['ccustname'];
			if (isset($_REQUEST['ccuststate'])) $ccuststate = $_REQUEST['ccuststate'];
			if (isset($_REQUEST['ccustcc'])) $ccustcc = $_REQUEST['ccustcc'];
			if (isset($_REQUEST['ccustemail'])) $ccustemail = $_REQUEST['ccustemail'];
			if (isset($_REQUEST['cproditem'])) $cproditem = $_REQUEST['cproditem'];
			if (isset($_REQUEST['cprodtitle'])) $cprodtitle = $_REQUEST['cprodtitle'];
			if (isset($_REQUEST['cprodtype'])) $cprodtype = $_REQUEST['cprodtype'];
			if (isset($_REQUEST['ctransaction'])) $ctransaction = $_REQUEST['ctransaction'];
			if (isset($_REQUEST['ctransaffiliate'])) $ctransaffiliate = $_REQUEST['ctransaffiliate'];
			if (isset($_REQUEST['ctransamount'])) $ctransamount = $_REQUEST['ctransamount'];
			if (isset($_REQUEST['ctranspaymentmethod'])) $ctranspaymentmethod = $_REQUEST['ctranspaymentmethod'];
			if (isset($_REQUEST['ctransvendor'])) $ctransvendor = $_REQUEST['ctransvendor'];
			if (isset($_REQUEST['ctransreceipt'])) $ctransreceipt = $_REQUEST['ctransreceipt'];
			if (isset($_REQUEST['cupsellreceipt'])) $cupsellreceipt = $_REQUEST['cupsellreceipt'];
			if (isset($_REQUEST['caffitid'])) $caffitid = $_REQUEST['caffitid'];
			if (isset($_REQUEST['cvendthru'])) $cvendthru = $_REQUEST['cvendthru'];
			if (isset($_REQUEST['cverify'])) $cverify = $_REQUEST['cverify'];
			if (isset($_REQUEST['ctranstime'])) $ctranstime = date('Y/m/d H:i:s', strtotime($_REQUEST['ctranstime']));
			
			//Save the payment informations
			$this->jvzoolistenermodel->CCustName = $ccustname;
			$this->jvzoolistenermodel->CCustState = $ccuststate;
			$this->jvzoolistenermodel->CCustCc = $ccustcc;
			$this->jvzoolistenermodel->CCustEmail = $ccustemail;
			$this->jvzoolistenermodel->CProdItem = $cproditem;
			$this->jvzoolistenermodel->CProdTitle = $cprodtitle;
			$this->jvzoolistenermodel->CProdType = $cprodtype;
			$this->jvzoolistenermodel->CTransaction = $ctransaction;
			$this->jvzoolistenermodel->CTransAffiliate = $ctransaffiliate;
			$this->jvzoolistenermodel->CTransAmount = $ctransamount;
			$this->jvzoolistenermodel->CTransPaymentMethod = $ctranspaymentmethod;
			$this->jvzoolistenermodel->CTransVendor = $ctransvendor;
			$this->jvzoolistenermodel->CUpsellReceipt = $cupsellreceipt;
			$this->jvzoolistenermodel->CAffitID = $caffitid;
			$this->jvzoolistenermodel->CVendThru = $cvendthru;
			$this->jvzoolistenermodel->CVerify = $cverify;
			$this->jvzoolistenermodel->CTransTime = $ctranstime;
			$this->jvzoolistenermodel->UpdatedOn = date('Y/m/d H:i:s', time());
			
			$ret = $this->jvzoolistenermodel->create(3);
		
			if (!$ret){
				$msg = "JvZoo - listenerJvZoo - Create Content Detail - ".$this->jvzoolistenermodel->Error;
				echo $msg;
				//Common::sendAdministrativeMail($msg);
				return;
			}
			
			//In case this is the first payment, create an Authorization to register on the system
			if (strtoupper($ctransaction) == 'SALE'){
				
				//set the bodyMessage and CreatedBy User
				$bodyEmail = null; 
				
				$profile = $this->ion_auth->where('username', $ccustemail)->users()->row(); //pass the code to profile
				//in case the user is new, create it.
				if (!$profile) {
										
					//$access = 1;
					
					//configure the products from jvzoo
					$fe = $this->config->item('fe', 'jvzoo');
					$oto1 = $this->config->item('oto1', 'jvzoo');
					$oto2 = $this->config->item('oto2', 'jvzoo');
					
					//Define the User Type according to the price of the payment
					if(strpos($fe, $this->jvzoolistenermodel->CProdItem) !== false) {
						$access = 1;
					}elseif(strpos($oto1, $this->jvzoolistenermodel->CProdItem) !== false) {
						$access = '1,2';
					}elseif(strpos($oto2, $this->jvzoolistenermodel->CProdItem) !== false) {
						$access = '1,2,3';
					}
					$pass = md5(uniqid(rand(0,5), true));
					$pass = (strlen($pass) > 8) ? substr($pass,0,7): $pass;					
					$this->ion_auth->register($ccustemail, 'nich9Nfc' , $ccustemail, array('first_name'=>$ccustname,'product_id'=>$this->jvzoolistenermodel->CProdItem ), array('2'), true, $access);
					$this->select();
				//in case the user already exists, update its access
				}else{
					
					$access = 1;
					
					//configure the products from jvzoo
					$fe = $this->config->item('fe', 'jvzoo');
					$oto1 = $this->config->item('oto1', 'jvzoo');
					$oto2 = $this->config->item('oto2', 'jvzoo');
					
					//Define the User Type according to the price of the payment
					if(strpos($fe, $this->jvzoolistenermodel->CProdItem) !== false) {
						$access = '1';
					}elseif(strpos($oto1, $this->jvzoolistenermodel->CProdItem) !== false) {
						$access = '2';
					}elseif(strpos($oto2, $this->jvzoolistenermodel->CProdItem) !== false) {
						$access = '3';
					}
					
					$pos = strpos(','.$profile->access, ','.$access);
					//var_dump($pos);
					
					if($pos === false) {
						$profile->access .= ','.$access;
						//update user
						$this->usermodel->updateUserAccess($profile->id, $profile->access ,$this->jvzoolistenermodel->CProdItem);
						$data ='';
				if($access=='2')
					{	
					//echo $access."inside upgrade1";
				$this->load->library('email'); 		
				
				$message = $this->load->view($this->config->item('email_templates', 'ion_auth').$this->config->item('email_upgrade', 'ion_auth'),  $data, true);
				
				//$message = 'Congrats you are successfully Upgraded!!';

				$this->email->clear();
				$this->email->from($this->config->item('admin_email', 'ion_auth'), $this->config->item('site_title', 'ion_auth'));
				$this->email->to($ccustemail);
				$this->email->subject($this->config->item('site_title', 'ion_auth') . ' - Pro');
				$this->email->message($message);
				
								if ($this->email->send() == TRUE)
							{
					//print(1);
							echo "Successfully upgraded for upgrade1 access";
								
								//$this->set_message('activation_email_successful');
								//return $id;
							}else{
								//die($this->email->print_debugger());
								//return FALSE;
								//print(2);
							}
					}						
				if($access=='3')
					{	
					//echo $access."inside upgrade1";
				$this->load->library('email'); 		
				
				$message = $this->load->view($this->config->item('email_templates', 'ion_auth').$this->config->item('email_upgrade2', 'ion_auth'), $data,  true);
				
				//$message = 'Congrats you are successfully Upgraded!!';

				$this->email->clear();
				$this->email->from($this->config->item('admin_email', 'ion_auth'), $this->config->item('site_title', 'ion_auth'));
				$this->email->to($ccustemail);
				$this->email->subject($this->config->item('site_title', 'ion_auth') . ' - White Label');
				$this->email->message($message);
				
								if ($this->email->send() == TRUE)
							{
					//print(1);
								echo "Successfully upgraded for upgrade2 access";
								
								//$this->set_message('activation_email_successful');
								//return $id;
							}else{
								//die($this->email->print_debugger());
								//return FALSE;
								//print(2);
							}
					}						
					}
				}
			}else if(strtoupper($ctransaction) == 'RFND'){
				$msg = "JvZoo - Refunded - <b>UserID</b>:".$ccustemail;
				//Common::sendAdministrativeMail($msg);
			}else if(strtoupper($ctransaction) == 'CGBK'){
				$msg = "JvZoo - ChargeBack  - <b>UserID</b>:".$ccustemail;
				//Common::sendAdministrativeMail($msg);
			}else if(strtoupper($ctransaction) == 'INSF'){
				$msg = "JvZoo - ECheck ChargeBack  - <b>UserID</b>:".$ccustemail;
				//Common::sendAdministrativeMail($msg);
			}else if(strtoupper($ctransaction) == 'CANCEL-REBILL'){
				$msg = "JvZoo - Cancellation of a Recurring Billing - <b>UserID</b>:".$ccustemail;
				//Common::sendAdministrativeMail($msg);
			}else if(strtoupper($ctransaction) == 'UNCANCEL-REBILL'){
				$msg = "JvZoo - Reversing the Cancellation of a Recurring Billing - <b>UserID</b>:".$ccustemail;
				//Common::sendAdministrativeMail($msg);
			}
			
		//	echo "IPN received successfully.";
		redirect('https://engageleads.leadpages.co/niche-evo-ty/');	
			exit;
			 
			// IPN message values depend upon the type of notification sent.
			// To loop through the &_POST array and print the NV pairs to the screen:
			//foreach($_REQUEST as $key => $value) {
			//	echo $key." = ". $value."<br>";
			//}
		} else {
			$msg = "JvZoo - listenerJvZoo - Invalid IPN call - <b>Data</b>:".$raw_post_data;
			//Common::sendAdministrativeMail($msg);
			// IPN invalid, log for manual investigation
			echo "JvZoo - listenerJvZoo - Invalid IPN call";
			return;
			//echo "The response from IPN was: <b>" .$res ."</b>";
		}
		return;
	}
	//cahange
	
	function select()
	{


			$this->auto();
			//$this->web();

	}
	
	//active compain
	function auto()
	{
		$a =$this->jvzoolistenermodel->CCustName;
		//echo $a;
		$b =$this->jvzoolistenermodel->CCustEmail;
	                     $c='https://engageleads.api-us1.com';
					$d ='edb44fd56a36641c0867a118c808c4e75e6c94859bcdbb8d91ee7c35561b106baf7d22c6'; 
					$e=81;
					$f=1;		
		//echo $c;
		//echo $d;

	
	try{
	$url=$c;
	$AppKey=$d;
	$name=$a;
	$email=$b;	
	$ListId=$e;
	//$Listname=param5;
	
	$params = array(

    // the API Key can be found on the "Your Settings" page under the "API" tab.
    // replace this with your API Key
    'api_key'      => $AppKey,

    // this is the action that adds a contact
    'api_action'   => 'contact_add',

    // define the type of output you wish to get back
    // possible values:
    // - 'xml'  :      you have to write your own XML parser
    // - 'json' :      data is returned in JSON format and can be decoded with
    //                 json_decode() function (included in PHP since 5.2.0)
    // - 'serialize' : data is returned in a serialized format and can be decoded with
    //                 a native unserialize() function
    'api_output'   => 'serialize',
);

// here we define the data we are posting in order to perform an update
$post = array(
    'email'                    => $email,
    'first_name'               => $name,
    'last_name'                => '',
    'phone'                    => '',
    'orgname'                  => '',
    'tags'                     => 'api',
    'ip4'                    => $_SERVER['REMOTE_ADDR'],

    // any custom fields
    //'field[345,0]'           => 'field value', // where 345 is the field ID
    //'field[%PERS_1%,0]'      => 'field value', // using the personalization tag instead (make sure to encode the key)

    // assign to lists:
    'p['.$ListId.']'                   => $ListId, // example list ID (REPLACE '123' WITH ACTUAL LIST ID, IE: p[5] = 5)
    'status['.$ListId.']'              => 1, // 1: active, 2: unsubscribed (REPLACE '123' WITH ACTUAL LIST ID, IE: status[5] = 1)
    //'form'          => 1001, // Subscription Form ID, to inherit those redirection settings
    //'noresponders[123]'      => 1, // uncomment to set "do not send any future responders"
    //'sdate[123]'             => '2009-12-07 06:00:00', // Subscribe date for particular list - leave out to use current date/time
    // use the folowing only if status=1
    'instantresponders['.$ListId.']' => 1, // set to 0 to if you don't want to sent instant autoresponders
    //'lastmessage[123]'       => 1, // uncomment to set "send the last broadcast campaign"

    //'p[]'                    => 345, // some additional lists?
    //'status[345]'            => 1, // some additional lists?
);
//print_r($params);
//print_r($post);
// This section takes the input fields and converts them to the proper format
$query = "";
foreach( $params as $key => $value ) $query .= $key . '=' . urlencode($value) . '&';
$query = rtrim($query, '& ');

// This section takes the input data and converts it to the proper format
$data = "";
foreach( $post as $key => $value ) $data .= $key . '=' . urlencode($value) . '&';
$data = rtrim($data, '& ');

// clean up the url
$url = rtrim($url, '/ ');
// This sample code uses the CURL library for php to establish a connection,
// submit your request, and show (print out) the response.

if ( !function_exists('curl_init') ) die('CURL not supported. (introduced in PHP 4.0.2)');

// If JSON is used, check if json_decode is present (PHP 5.2.0+)
if ( $params['api_output'] == 'json' && !function_exists('json_decode') ) {
    die('JSON not supported. (introduced in PHP 5.2.0)');
}

// define a final API request - GET
$api = $url . '/admin/api.php?' . $query;

$request = curl_init($api); // initiate curl object
curl_setopt($request, CURLOPT_HEADER, 0); // set to 0 to eliminate header info from response
curl_setopt($request, CURLOPT_RETURNTRANSFER, 1); // Returns response data instead of TRUE(1)
curl_setopt($request, CURLOPT_POSTFIELDS, $data); // use HTTP POST to send form data
curl_setopt($request, CURLOPT_SSL_VERIFYPEER, FALSE); // uncomment if you get no gateway response and are using HTTPS
curl_setopt($request, CURLOPT_FOLLOWLOCATION, true);

$response = (string)curl_exec($request); // execute curl post and store results in $response

// additional options may be required depending upon your server configuration
// you can find documentation on curl options at http://www.php.net/curl_setopt
curl_close($request); // close curl object

if ( !$response ) {
    die('Nothing was returned. Do you have a connection to Email Marketing server?');
}

// This line takes the response and breaks it into an array using:
// JSON decoder
//$result = json_decode($response);
// unserializer
$result = unserialize($response);
// XML parser...
// ...

// Result info that is always returned
//echo 'Result: ' . ( $result['result_code'] ? 'SUCCESS' : 'FAILED' ) . '<br />';
//echo 'Message: ' . $result['result_message'] . '<br />';
if($result['result_code']==0){
	
	echo json_encode(array('error'=>41, 'message'=>$result['result_message']));
}else{
	
	echo json_encode(array('success'=>1));
}

				
			//exit;
}
catch(Exception $e){
			echo json_encode(array('error'=>41, 'message'=>'Error subscribing on Active Campaign. '. $e->getMessage()));
			exit;
			
}
	

	}
		//mailer Goto Webinar
		public function web(){
			

                     $c='1088137544512608774';
					$d ='qOOmsZ9Vl8WYGsHnE3AkGAB0Aq2c'; 
					$e='7668152796342501377';
					$f=1;

		//echo $c;
		//echo $d;

	
		//organizerkey = 3170356;
		//webinarkey = 850278193;
		//oauthtoken = rnxzREe75e800YrCJAygGUXyM9Xg;
		
		
		$data = array(
		
  "firstName"=> $this->jvzoolistenermodel->CCustName,
  "lastName"=> $this->jvzoolistenermodel->CCustName,
  "email"=> $this->jvzoolistenermodel->CCustEmail,
 
    
  

		);
		$organizerkey = $c;
		$accessToken = $d;
		$webinarkey = $e;
		
		//print_r($data);
		//exit;

		$gotoWebinar = new GotoWebinar();
		$register = $gotoWebinar->SubmitGotoWebinar($data, $organizerkey, $webinarkey, $accessToken);
		//$test = json_decode($register);
		//print_r($register);
		//exit;
		
		if ($register === false){
			echo json_encode(array('error'=>41, 'message'=>'Error subscribing on GoToWebinar.'));
			exit;
		}else{
			$pos = strpos($register, '{"joinUrl"');
			$pos1 = strpos($register, '{"registrantKey"');
			if ($pos === false && $pos1 === false) {
				echo json_encode(array('error'=>42, 'message'=>'Error subscribing on GoToWebinar.'));
				exit;
			}else{
				if ($pos) $register = substr($register, $pos);
				if ($pos1) $register = substr($register, $pos1);
				
				$result = json_decode($register);
				//print_r($result);
				echo json_encode(array('success'=>1));
				//exit; 
			}
		}
	
			
		}

	//log the user out
	function logout()
	{

			$user = $this->ion_auth->user()->row();
			$userID = $user->id;
						if((strpos($this->session->userdata('access'), ',3') == true) )
			{$parent_user_id = $userID;}
		else{
		$result = mysqli_query($this->db->conn_id,'select parent_user_id from users where id="'.$userID.'"');
						while($row = mysqli_fetch_array($result)){
			
				if(array_key_exists('parent_user_id', $row))
				{
				$parent_user_id = $row['parent_user_id'];
				}
				else
				{
					$parent_user_id = '';
				}
						}
		}
			 
				
		$this->data['title'] = "Logout";

		//log the user out
		$logout = $this->ion_auth->logout();

		//redirect them to the login page
		$this->session->set_flashdata('message', $this->ion_auth->messages());
		redirect('auth/login?wl='.$parent_user_id, 'refresh');
	}

	//change password
	function change_password()
	{
		$this->form_validation->set_rules('old', $this->lang->line('change_password_validation_old_password_label'), 'required');
		$this->form_validation->set_rules('new', $this->lang->line('change_password_validation_new_password_label'), 'required|min_length[' . $this->config->item('min_password_length', 'ion_auth') . ']|max_length[' . $this->config->item('max_password_length', 'ion_auth') . ']|matches[new_confirm]');
		$this->form_validation->set_rules('new_confirm', $this->lang->line('change_password_validation_new_password_confirm_label'), 'required');

		if (!$this->ion_auth->logged_in())
		{
			redirect('auth/login', 'refresh');
		}

		$user = $this->ion_auth->user()->row();

		if ($this->form_validation->run() == false)
		{
			//display the form
			//set the flash data error message if there is one
			$this->data['message'] = (validation_errors()) ? validation_errors() : $this->session->flashdata('message');

			$this->data['min_password_length'] = $this->config->item('min_password_length', 'ion_auth');
			$this->data['old_password'] = array(
				'name' => 'old',
				'id'   => 'old',
				'type' => 'password',
			);
			$this->data['new_password'] = array(
				'name' => 'new',
				'id'   => 'new',
				'type' => 'password',
				'pattern' => '^.{'.$this->data['min_password_length'].'}.*$',
			);
			$this->data['new_password_confirm'] = array(
				'name' => 'new_confirm',
				'id'   => 'new_confirm',
				'type' => 'password',
				'pattern' => '^.{'.$this->data['min_password_length'].'}.*$',
			);
			$this->data['user_id'] = array(
				'name'  => 'user_id',
				'id'    => 'user_id',
				'type'  => 'hidden',
				'value' => $user->id,
			);

			//render
			$this->_render_page('auth/change_password', $this->data);
		}
		else
		{
			$identity = $this->session->userdata('identity');

			$change = $this->ion_auth->change_password($identity, $this->input->post('old'), $this->input->post('new'));

			if ($change)
			{
				//if the password was successfully changed
				$this->session->set_flashdata('message', $this->ion_auth->messages());
				$this->logout();
			}
			else
			{
				$this->session->set_flashdata('message', $this->ion_auth->errors());
				redirect('auth/change_password', 'refresh');
			}
		}
	}

	//forgot password
	function forgot_password()
	{
		$this->form_validation->set_rules('email', $this->lang->line('forgot_password_validation_email_label'), 'required|valid_email');
		if ($this->form_validation->run() == false)
		{
			//setup the input
			$this->data['email'] = array('name' => 'email',
				'id' => 'email',
			);

			if ( $this->config->item('identity', 'ion_auth') == 'username' ){
				$this->data['identity_label'] = $this->lang->line('forgot_password_username_identity_label');
			}
			else
			{
				$this->data['identity_label'] = $this->lang->line('forgot_password_email_identity_label');
			}

			//set any errors and display the form
			$this->data['message'] = (validation_errors()) ? validation_errors() : $this->session->flashdata('message');
			$this->_render_page('auth/forgot_password', $this->data);
		}
		else
		{
			// get identity from username or email
			if ( $this->config->item('identity', 'ion_auth') == 'username' ){
				$identity = $this->ion_auth->where('username', strtolower($this->input->post('email')))->users()->row();
			}
			else
			{
				$identity = $this->ion_auth->where('email', strtolower($this->input->post('email')))->users()->row();
			}
	            	if(empty($identity)) {
		        	$this->ion_auth->set_message('forgot_password_email_not_found');
		                $this->session->set_flashdata('message', $this->ion_auth->messages());
                		redirect("auth/forgot_password", 'refresh');
            		}

			//run the forgotten password method to email an activation code to the user
			$forgotten = $this->ion_auth->forgotten_password($identity->{$this->config->item('identity', 'ion_auth')});

			if ($forgotten)
			{
				//if there were no errors
				$this->session->set_flashdata('message', $this->ion_auth->messages());
				redirect("auth/login", 'refresh'); //we should display a confirmation page here instead of the login page
			}
			else
			{
				$this->session->set_flashdata('message', $this->ion_auth->errors());
				redirect("auth/forgot_password", 'refresh');
			}
		}
	}

	//reset password - final step for forgotten password
	public function reset_password($code = NULL)
	{
		if (!$code)
		{
			show_404();
		}

		$user = $this->ion_auth->forgotten_password_check($code);

		if ($user)
		{
			//if the code is valid then display the password reset form

			$this->form_validation->set_rules('new', $this->lang->line('reset_password_validation_new_password_label'), 'required|min_length[' . $this->config->item('min_password_length', 'ion_auth') . ']|max_length[' . $this->config->item('max_password_length', 'ion_auth') . ']|matches[new_confirm]');
			$this->form_validation->set_rules('new_confirm', $this->lang->line('reset_password_validation_new_password_confirm_label'), 'required');

			if ($this->form_validation->run() == false)
			{
				//display the form

				//set the flash data error message if there is one
				$this->data['message'] = (validation_errors()) ? validation_errors() : $this->session->flashdata('message');

				$this->data['min_password_length'] = $this->config->item('min_password_length', 'ion_auth');
				$this->data['new_password'] = array(
					'name' => 'new',
					'id'   => 'new',
				'type' => 'password',
					'pattern' => '^.{'.$this->data['min_password_length'].'}.*$',
				);
				$this->data['new_password_confirm'] = array(
					'name' => 'new_confirm',
					'id'   => 'new_confirm',
					'type' => 'password',
					'pattern' => '^.{'.$this->data['min_password_length'].'}.*$',
				);
				$this->data['user_id'] = array(
					'name'  => 'user_id',
					'id'    => 'user_id',
					'type'  => 'hidden',
					'value' => $user->id,
				);
				$this->data['csrf'] = $this->_get_csrf_nonce();
				$this->data['code'] = $code;

				//render
				$this->_render_page('auth/reset_password', $this->data);
			}
			else
			{
				// do we have a valid request?
				if ($this->_valid_csrf_nonce() === FALSE || $user->id != $this->input->post('user_id'))
				{

					//something fishy might be up
					$this->ion_auth->clear_forgotten_password_code($code);

					show_error($this->lang->line('error_csrf'));

				}
				else
				{
					// finally change the password
					$identity = $user->{$this->config->item('identity', 'ion_auth')};

					$change = $this->ion_auth->reset_password($identity, $this->input->post('new'));

					if ($change)
					{
						//if the password was successfully changed
						$this->session->set_flashdata('message', $this->ion_auth->messages());
						$this->logout();
					}
					else
					{
						$this->session->set_flashdata('message', $this->ion_auth->errors());
						redirect('auth/reset_password/' . $code, 'refresh');
					}
				}
			}
		}
		else
		{
			//if the code is invalid then send them back to the forgot password page
			$this->session->set_flashdata('message', $this->ion_auth->errors());
			redirect("auth/forgot_password", 'refresh');
		}
	}


	//activate the user
	function activate($id, $code=false)
	{
		if ($code !== false)
		{
			$activation = $this->ion_auth->activate($id, $code);
		}
		else if ($this->ion_auth->is_admin())
		{
			$activation = $this->ion_auth->activate($id);
		}

		if ($activation)
		{
			//redirect them to the auth page
			$this->session->set_flashdata('message', $this->ion_auth->messages());
			redirect("auth", 'refresh');
		}
		else
		{
			//redirect them to the forgot password page
			$this->session->set_flashdata('message', $this->ion_auth->errors());
			redirect("auth/forgot_password", 'refresh');
		}
	}

	//deactivate the user
	function deactivate($id = NULL)
	{
		$id = (int) $id;

		$this->load->library('form_validation');
		$this->form_validation->set_rules('confirm', $this->lang->line('deactivate_validation_confirm_label'), 'required');
		$this->form_validation->set_rules('id', $this->lang->line('deactivate_validation_user_id_label'), 'required|alpha_numeric');

		if ($this->form_validation->run() == FALSE)
		{
			// insert csrf check
			$this->data['csrf'] = $this->_get_csrf_nonce();
			$this->data['user'] = $this->ion_auth->user($id)->row();

			$this->_render_page('auth/deactivate_user', $this->data);
		}
		else
		{
			// do we really want to deactivate?
			if ($this->input->post('confirm') == 'yes')
			{
				// do we have a valid request?
				if ($this->_valid_csrf_nonce() === FALSE || $id != $this->input->post('id'))
				{
					show_error($this->lang->line('error_csrf'));
				}

				// do we have the right userlevel?
				if ($this->ion_auth->logged_in() && $this->ion_auth->is_admin())
				{
					$this->ion_auth->deactivate($id);
				}
			}

			//redirect them back to the auth page
			redirect('auth', 'refresh');
		}
	}

	//create a new user
	function create_user()
	{
		$this->data['title'] = "Create User";

		if (!$this->ion_auth->logged_in() || !$this->ion_auth->is_admin())
		{
			redirect('auth', 'refresh');
		}

		$tables = $this->config->item('tables','ion_auth');

		//validate form input
		$this->form_validation->set_rules('first_name', $this->lang->line('create_user_validation_fname_label'), 'required|xss_clean');
		$this->form_validation->set_rules('last_name', $this->lang->line('create_user_validation_lname_label'), 'required|xss_clean');
		$this->form_validation->set_rules('email', $this->lang->line('create_user_validation_email_label'), 'required|valid_email|is_unique['.$tables['users'].'.email]');
		$this->form_validation->set_rules('phone', $this->lang->line('create_user_validation_phone_label'), 'required|xss_clean');
		$this->form_validation->set_rules('company', $this->lang->line('create_user_validation_company_label'), 'required|xss_clean');
		$this->form_validation->set_rules('password', $this->lang->line('create_user_validation_password_label'), 'required|min_length[' . $this->config->item('min_password_length', 'ion_auth') . ']|max_length[' . $this->config->item('max_password_length', 'ion_auth') . ']|matches[password_confirm]');
		$this->form_validation->set_rules('password_confirm', $this->lang->line('create_user_validation_password_confirm_label'), 'required');

		if ($this->form_validation->run() == true)
		{
			$username = strtolower($this->input->post('first_name')) . ' ' . strtolower($this->input->post('last_name'));
			$email    = strtolower($this->input->post('email'));
			$password = $this->input->post('password');

			$additional_data = array(
				'first_name' => $this->input->post('first_name'),
				'last_name'  => $this->input->post('last_name'),
				'company'    => $this->input->post('company'),
				'phone'      => $this->input->post('phone'),
			);
		}
		if ($this->form_validation->run() == true && $this->ion_auth->register($username, $password, $email, $additional_data))
		{
			//check to see if we are creating the user
			//redirect them back to the admin page
			$this->session->set_flashdata('message', $this->ion_auth->messages());
			redirect("auth", 'refresh');
		}
		else
		{
			//display the create user form
			//set the flash data error message if there is one
			$this->data['message'] = (validation_errors() ? validation_errors() : ($this->ion_auth->errors() ? $this->ion_auth->errors() : $this->session->flashdata('message')));

			$this->data['first_name'] = array(
				'name'  => 'first_name',
				'id'    => 'first_name',
				'type'  => 'text',
				'value' => $this->form_validation->set_value('first_name'),
			);
			$this->data['last_name'] = array(
				'name'  => 'last_name',
				'id'    => 'last_name',
				'type'  => 'text',
				'value' => $this->form_validation->set_value('last_name'),
			);
			$this->data['email'] = array(
				'name'  => 'email',
				'id'    => 'email',
				'type'  => 'text',
				'value' => $this->form_validation->set_value('email'),
			);
			$this->data['company'] = array(
				'name'  => 'company',
				'id'    => 'company',
				'type'  => 'text',
				'value' => $this->form_validation->set_value('company'),
			);
			$this->data['phone'] = array(
				'name'  => 'phone',
				'id'    => 'phone',
				'type'  => 'text',
				'value' => $this->form_validation->set_value('phone'),
			);
			$this->data['password'] = array(
				'name'  => 'password',
				'id'    => 'password',
				'type'  => 'password',
				'value' => $this->form_validation->set_value('password'),
			);
			$this->data['password_confirm'] = array(
				'name'  => 'password_confirm',
				'id'    => 'password_confirm',
				'type'  => 'password',
				'value' => $this->form_validation->set_value('password_confirm'),
			);

			$this->_render_page('auth/create_user', $this->data);
		}
	}

	//edit a user
	function edit_user($id)
	{
		$this->data['title'] = "Edit User";

		if (!$this->ion_auth->logged_in() || (!$this->ion_auth->is_admin() && !($this->ion_auth->user()->row()->id == $id)))
		{
			redirect('auth', 'refresh');
		}

		$user = $this->ion_auth->user($id)->row();
		$groups=$this->ion_auth->groups()->result_array();
		$currentGroups = $this->ion_auth->get_users_groups($id)->result();

		//validate form input
		$this->form_validation->set_rules('first_name', $this->lang->line('edit_user_validation_fname_label'), 'required|xss_clean');
		$this->form_validation->set_rules('last_name', $this->lang->line('edit_user_validation_lname_label'), 'required|xss_clean');
		$this->form_validation->set_rules('phone', $this->lang->line('edit_user_validation_phone_label'), 'required|xss_clean');
		$this->form_validation->set_rules('company', $this->lang->line('edit_user_validation_company_label'), 'required|xss_clean');
		$this->form_validation->set_rules('groups', $this->lang->line('edit_user_validation_groups_label'), 'xss_clean');

		if (isset($_POST) && !empty($_POST))
		{
			// do we have a valid request?
			if ($this->_valid_csrf_nonce() === FALSE || $id != $this->input->post('id'))
			{
				show_error($this->lang->line('error_csrf'));
			}

			$data = array(
				'first_name' => $this->input->post('first_name'),
				'last_name'  => $this->input->post('last_name'),
				'company'    => $this->input->post('company'),
				'phone'      => $this->input->post('phone'),
			);

			// Only allow updating groups if user is admin
			if ($this->ion_auth->is_admin())
			{
				//Update the groups user belongs to
				$groupData = $this->input->post('groups');

				if (isset($groupData) && !empty($groupData)) {

					$this->ion_auth->remove_from_group('', $id);

					foreach ($groupData as $grp) {
						$this->ion_auth->add_to_group($grp, $id);
					}

				}
			}

			//update the password if it was posted
			if ($this->input->post('password'))
			{
				$this->form_validation->set_rules('password', $this->lang->line('edit_user_validation_password_label'), 'required|min_length[' . $this->config->item('min_password_length', 'ion_auth') . ']|max_length[' . $this->config->item('max_password_length', 'ion_auth') . ']|matches[password_confirm]');
				$this->form_validation->set_rules('password_confirm', $this->lang->line('edit_user_validation_password_confirm_label'), 'required');

				$data['password'] = $this->input->post('password');
			}

			if ($this->form_validation->run() === TRUE)
			{
				$this->ion_auth->update($user->id, $data);

				//check to see if we are creating the user
				//redirect them back to the admin page
				$this->session->set_flashdata('message', "User Saved");
				if ($this->ion_auth->is_admin())
				{
					redirect('auth', 'refresh');
				}
				else
				{
					redirect('/', 'refresh');
				}
			}
		}

		//display the edit user form
		$this->data['csrf'] = $this->_get_csrf_nonce();

		//set the flash data error message if there is one
		$this->data['message'] = (validation_errors() ? validation_errors() : ($this->ion_auth->errors() ? $this->ion_auth->errors() : $this->session->flashdata('message')));

		//pass the user to the view
		$this->data['user'] = $user;
		$this->data['groups'] = $groups;
		$this->data['currentGroups'] = $currentGroups;

		$this->data['first_name'] = array(
			'name'  => 'first_name',
			'id'    => 'first_name',
			'type'  => 'text',
			'value' => $this->form_validation->set_value('first_name', $user->first_name),
		);
		$this->data['last_name'] = array(
			'name'  => 'last_name',
			'id'    => 'last_name',
			'type'  => 'text',
			'value' => $this->form_validation->set_value('last_name', $user->last_name),
		);
		$this->data['company'] = array(
			'name'  => 'company',
			'id'    => 'company',
			'type'  => 'text',
			'value' => $this->form_validation->set_value('company', $user->company),
		);
		$this->data['phone'] = array(
			'name'  => 'phone',
			'id'    => 'phone',
			'type'  => 'text',
			'value' => $this->form_validation->set_value('phone', $user->phone),
		);
		$this->data['password'] = array(
			'name' => 'password',
			'id'   => 'password',
			'type' => 'password'
		);
		$this->data['password_confirm'] = array(
			'name' => 'password_confirm',
			'id'   => 'password_confirm',
			'type' => 'password'
		);

		$this->_render_page('auth/edit_user', $this->data);
	}

	// create a new group
	function create_group()
	{
		$this->data['title'] = $this->lang->line('create_group_title');

		if (!$this->ion_auth->logged_in() || !$this->ion_auth->is_admin())
		{
			redirect('auth', 'refresh');
		}

		//validate form input
		$this->form_validation->set_rules('group_name', $this->lang->line('create_group_validation_name_label'), 'required|alpha_dash|xss_clean');
		$this->form_validation->set_rules('description', $this->lang->line('create_group_validation_desc_label'), 'xss_clean');

		if ($this->form_validation->run() == TRUE)
		{
			$new_group_id = $this->ion_auth->create_group($this->input->post('group_name'), $this->input->post('description'));
			if($new_group_id)
			{
				// check to see if we are creating the group
				// redirect them back to the admin page
				$this->session->set_flashdata('message', $this->ion_auth->messages());
				redirect("auth", 'refresh');
			}
		}
		else
		{
			//display the create group form
			//set the flash data error message if there is one
			$this->data['message'] = (validation_errors() ? validation_errors() : ($this->ion_auth->errors() ? $this->ion_auth->errors() : $this->session->flashdata('message')));

			$this->data['group_name'] = array(
				'name'  => 'group_name',
				'id'    => 'group_name',
				'type'  => 'text',
				'value' => $this->form_validation->set_value('group_name'),
			);
			$this->data['description'] = array(
				'name'  => 'description',
				'id'    => 'description',
				'type'  => 'text',
				'value' => $this->form_validation->set_value('description'),
			);

			$this->_render_page('auth/create_group', $this->data);
		}
	}

	//edit a group
	function edit_group($id)
	{
		// bail if no group id given
		if(!$id || empty($id))
		{
			redirect('auth', 'refresh');
		}

		$this->data['title'] = $this->lang->line('edit_group_title');

		if (!$this->ion_auth->logged_in() || !$this->ion_auth->is_admin())
		{
			redirect('auth', 'refresh');
		}

		$group = $this->ion_auth->group($id)->row();

		//validate form input
		$this->form_validation->set_rules('group_name', $this->lang->line('edit_group_validation_name_label'), 'required|alpha_dash|xss_clean');
		$this->form_validation->set_rules('group_description', $this->lang->line('edit_group_validation_desc_label'), 'xss_clean');

		if (isset($_POST) && !empty($_POST))
		{
			if ($this->form_validation->run() === TRUE)
			{
				$group_update = $this->ion_auth->update_group($id, $_POST['group_name'], $_POST['group_description']);

				if($group_update)
				{
					$this->session->set_flashdata('message', $this->lang->line('edit_group_saved'));
				}
				else
				{
					$this->session->set_flashdata('message', $this->ion_auth->errors());
				}
				redirect("auth", 'refresh');
			}
		}

		//set the flash data error message if there is one
		$this->data['message'] = (validation_errors() ? validation_errors() : ($this->ion_auth->errors() ? $this->ion_auth->errors() : $this->session->flashdata('message')));

		//pass the user to the view
		$this->data['group'] = $group;

		$this->data['group_name'] = array(
			'name'  => 'group_name',
			'id'    => 'group_name',
			'type'  => 'text',
			'value' => $this->form_validation->set_value('group_name', $group->name),
		);
		$this->data['group_description'] = array(
			'name'  => 'group_description',
			'id'    => 'group_description',
			'type'  => 'text',
			'value' => $this->form_validation->set_value('group_description', $group->description),
		);

		$this->_render_page('auth/edit_group', $this->data);
	}


	function _get_csrf_nonce()
	{
		$this->load->helper('string');
		$key   = random_string('alnum', 8);
		$value = random_string('alnum', 20);
		$this->session->set_flashdata('csrfkey', $key);
		$this->session->set_flashdata('csrfvalue', $value);

		return array($key => $value);
	}

	function _valid_csrf_nonce()
	{
		if ($this->input->post($this->session->flashdata('csrfkey')) !== FALSE &&
			$this->input->post($this->session->flashdata('csrfkey')) == $this->session->flashdata('csrfvalue'))
		{
			return TRUE;
		}
		else
		{
			return FALSE;
		}
	}

	function _render_page($view, $data=null, $render=false)
	{

		$this->viewdata = (empty($data)) ? $this->data: $data;

		$view_html = $this->load->view($view, $this->viewdata, $render);

		if (!$render) return $view_html;
	}

}
