<?php



    class TextSpinner
    {
        private static $_instance;

        private $_file;
        private $_data;
        private $_found;

        public function __construct($lang = "en")
        {
            $this->_file = $this->getFile($lang);
        }

        public function spin($data, $maxWordsToSpin = 0)
        {
            $maxWordsToSpin = intval($maxWordsToSpin);
            $this->_found = array();
            $this->_data = $data;

            // extract html tags
            $htmlTags = $this->extract("/<[^<>]+>/is");

            // extract shortcodes
            $shortCodes = $this->extract("/\[.*?\]/s");

            $tagsToExclude = array_merge($htmlTags, $shortCodes);
            foreach ($tagsToExclude as $tag) {
                $this->_data = str_replace($tag, '(' . md5($tag) . ')', $this->_data);
            }

            foreach ($this->_file as $line) {
                $this->analyzeLine($line);
                if($maxWordsToSpin != 0 && count($this->_found) >= $maxWordsToSpin) {
                    break;
                }
            }

            foreach ($tagsToExclude as $tag) {
                $this->_data = str_replace('(' . md5($tag) . ')', $tag, $this->_data);
            }

            if (count($this->_found) != 0) {
                foreach ($this->_found as $key => $val) {
                    $this->_data = str_replace($key, " " . $val, $this->_data);
                }
            }

            return $this->_data;
        }

        public function getData()
        {
            return $this->_data;
        }



        public static function getInstance()
        {
            if (!self::$_instance) {
                self::$_instance = new self();
            }

            return self::$_instance;
        }


        private function extract($pattern)
        {
            $matches = array();
            preg_match_all($pattern, $this->_data, $matches, PREG_PATTERN_ORDER);
            return $matches[0];
        }

        private function getFile($lang)
        {
            return file(dirname(__FILE__) . "/data/" . $lang . "_data.txt");
        }

        private function analyzeLine($line)
        {
            $synonyms = explode('|', $line);

            $cnt = count($synonyms);
            for ($i = 0; $i < $cnt; $i++) {
                $word = $synonyms[$i];
                if ($this->analyzeWord($word)) {
                    $rand = $i;
                    while ($rand == $i) {
                        $rand = rand(0, $cnt - 1);
                    }

                    $newWord = $synonyms[$rand];
                    $firstChar = substr($newWord, 0, 1);
                    if (ctype_upper($firstChar) || is_int($firstChar)) {
                        $this->replaceWord($word, $newWord);
                    } else {
                        $this->replaceWord($word, $newWord, true);

                        if (!is_numeric($word)) {
                            $word = ucfirst($word);
                            $newWord = ucfirst($newWord);
                            $this->replaceWord($word, $newWord);
                        }
                    }

                    break;
                }
            }
        }

        private function replaceWord($oldWord, $newWord, $caseSensitive = false)
        {
            $wordMD5 = md5($oldWord);
            $this->_found[$wordMD5] = str_replace(array("\n", "\r"), '', $newWord);
            $pattern = '/\b\s' . $oldWord . '\b/' . ($caseSensitive ? 'u' : 'i');
            $this->_data = preg_replace($pattern, $wordMD5, $this->_data);
        }
//
//        private function replaceWordCaseSensitive($oldWord, $newWord)
//        {
//            $wordMD5 = md5($oldWord);
//            $this->_found[$wordMD5] = str_replace(array("\n", "\r"), '', $newWord);
//            $this->_data = preg_replace('/\b\s' . $oldWord . '\b/u', $wordMD5, $this->_data);
//        }

        private function analyzeWord($word)
        {
            if (trim($word) != '') {
                $word2 = str_replace('/', '\/', $word);
                return preg_match('/\b\s' . $word2 . '\b/i', $this->_data);
            }
        }
    }

$spinObject = new TextSpinner();

$data = $_POST['article'];
$no_of_words = $_POST['no_of_words'];

echo $spinObject->spin($data, $no_of_words);